package sun.security.rsa;

import sun.security.jca.JCAUtil;
import java.util.WeakHashMap;
import java.security.interfaces.RSAPrivateCrtKey;
import java.security.interfaces.RSAPrivateKey;
import javax.crypto.BadPaddingException;
import java.security.interfaces.RSAPublicKey;
import java.security.interfaces.RSAKey;
import java.math.BigInteger;
import java.util.Map;

public final class RSACore
{
    private static final boolean ENABLE_BLINDING = true;
    private static final Map<BigInteger, RSACore.BlindingParameters> blindingCache;
    
    private RSACore() {
    }
    
    public static int getByteLength(BigInteger bigInteger) {
        return bigInteger.bitLength() + 7 >> 3;
    }
    
    public static int getByteLength(RSAKey rsaKey) {
        return getByteLength(rsaKey.getModulus());
    }
    
    public static byte[] convert(byte[] array, int n, int n2) {
        if (n == 0 && n2 == array.length) {
            return array;
        }
        byte[] array2 = new byte[n2];
        System.arraycopy(array, n, array2, 0, n2);
        return array2;
    }
    
    public static byte[] rsa(byte[] array, RSAPublicKey rsaPublicKey) throws BadPaddingException {
        return crypt(array, rsaPublicKey.getModulus(), rsaPublicKey.getPublicExponent());
    }
    
    @Deprecated
    public static byte[] rsa(byte[] array, RSAPrivateKey rsaPrivateKey) throws BadPaddingException {
        return rsa(array, rsaPrivateKey, true);
    }
    
    public static byte[] rsa(byte[] array, RSAPrivateKey rsaPrivateKey, boolean b) throws BadPaddingException {
        if ((rsaPrivateKey instanceof RSAPrivateCrtKey)) {
            return crtCrypt(array, (RSAPrivateCrtKey)rsaPrivateKey, b);
        }
        return priCrypt(array, rsaPrivateKey.getModulus(), rsaPrivateKey.getPrivateExponent());
    }
    
    private static byte[] crypt(byte[] array, BigInteger m, BigInteger exponent) throws BadPaddingException {
        return toByteArray(parseMsg(array, m).modPow(exponent, m), getByteLength(m));
    }
    
    private static byte[] priCrypt(byte[] array, BigInteger m, BigInteger exponent) throws BadPaddingException {
        BigInteger msg = parseMsg(array, m);
        RSACore.BlindingRandomPair blindingRandomPair = getBlindingRandomPair(null, exponent, m);
        return toByteArray(msg.multiply(blindingRandomPair.u).mod(m).modPow(exponent, m).multiply(blindingRandomPair.v).mod(m), getByteLength(m));
    }
    
    private static byte[] crtCrypt(byte[] array, RSAPrivateCrtKey rsaPrivateCrtKey, boolean b) throws BadPaddingException {
        BigInteger modulus = rsaPrivateCrtKey.getModulus();
        BigInteger msg = parseMsg(array, modulus);
        BigInteger primeP = rsaPrivateCrtKey.getPrimeP();
        BigInteger primeQ = rsaPrivateCrtKey.getPrimeQ();
        BigInteger primeExponentP = rsaPrivateCrtKey.getPrimeExponentP();
        BigInteger primeExponentQ = rsaPrivateCrtKey.getPrimeExponentQ();
        BigInteger crtCoefficient = rsaPrivateCrtKey.getCrtCoefficient();
        BigInteger publicExponent = rsaPrivateCrtKey.getPublicExponent();
        RSACore.BlindingRandomPair blindingRandomPair = getBlindingRandomPair(publicExponent, rsaPrivateCrtKey.getPrivateExponent(), modulus);
        BigInteger mod = msg.multiply(blindingRandomPair.u).mod(modulus);
        BigInteger modPow = mod.modPow(primeExponentP, primeP);
        BigInteger modPow2 = mod.modPow(primeExponentQ, primeQ);
        BigInteger bigInteger = modPow.subtract(modPow2);
        if (bigInteger.signum() < 0) {
            bigInteger = bigInteger.add(primeP);
        }
        BigInteger mod2 = bigInteger.multiply(crtCoefficient).mod(primeP).multiply(primeQ).add(modPow2).multiply(blindingRandomPair.v).mod(modulus);
        if (b && !msg.equals(mod2.modPow(publicExponent, modulus))) {
            throw new BadPaddingException("RSA private key operation failed");
        }
        return toByteArray(mod2, getByteLength(modulus));
    }
    
    private static BigInteger parseMsg(byte[] magnitude, BigInteger val) throws BadPaddingException {
        BigInteger bigInteger = new BigInteger(1, magnitude);
        if (bigInteger.compareTo(val) >= 0) {
            throw new BadPaddingException("Message is larger than modulus");
        }
        return bigInteger;
    }
    
    private static byte[] toByteArray(BigInteger bigInteger, int n) {
        byte[] byteArray = bigInteger.toByteArray();
        int length = byteArray.length;
        if (length == n) {
            return byteArray;
        }
        if (length == n + 1 && byteArray[0] == 0) {
            byte[] array2 = new byte[n];
            System.arraycopy(byteArray, 1, array2, 0, n);
            return array2;
        }
        assert length < n;
        byte[] array = new byte[n];
        System.arraycopy(byteArray, 0, array, n - length, length);
        return array;
    }
    
    private static RSACore.BlindingRandomPair getBlindingRandomPair(BigInteger bigInteger, BigInteger bigInteger2, BigInteger bigInteger3) {
        RSACore.BlindingParameters value = null;
        synchronized (RSACore.blindingCache) {
            value = (RSACore.BlindingParameters)RSACore.blindingCache.get(bigInteger3);
        }
        if (value == null) {
            value = new BlindingParameters(bigInteger, bigInteger2, bigInteger3);
            synchronized (RSACore.blindingCache) {
                RSACore.blindingCache.putIfAbsent(bigInteger3, value);
            }
        }
        RSACore.BlindingRandomPair blindingRandomPair = value.getBlindingRandomPair(bigInteger, bigInteger2, bigInteger3);
        if (blindingRandomPair == null) {
            RSACore.BlindingParameters value2 = new BlindingParameters(bigInteger, bigInteger2, bigInteger3);
            synchronized (RSACore.blindingCache) {
                RSACore.blindingCache.replace(bigInteger3, value2);
            }
            blindingRandomPair = value2.getBlindingRandomPair(bigInteger, bigInteger2, bigInteger3);
            return blindingRandomPair;
        }
        return blindingRandomPair;
    }
    
    static {
        blindingCache = new WeakHashMap();
    }
    
    private static final class BlindingRandomPair
    {
        final BigInteger u;
        final BigInteger v;
        
        BlindingRandomPair(BigInteger u, BigInteger v) {
            this.u = u;
            this.v = v;
        }
    }
    
    private static final class BlindingParameters
    {
        private static final BigInteger BIG_TWO;
        private final BigInteger e;
        private final BigInteger d;
        private BigInteger u;
        private BigInteger v;
        
        BlindingParameters(BigInteger bigInteger, BigInteger bigInteger2, BigInteger bigInteger3) {
            this.u = null;
            this.v = null;
            this.e = bigInteger;
            this.d = bigInteger2;
            this.u = new BigInteger(bigInteger3.bitLength(), JCAUtil.getSecureRandom()).mod(bigInteger3);
            if (this.u.equals(BigInteger.ZERO)) {
                this.u = BigInteger.ONE;
            }
            try {
                this.v = this.u.modInverse(bigInteger3);
            }
            catch (ArithmeticException ex) {
                this.u = BigInteger.ONE;
                this.v = BigInteger.ONE;
            }
            if (bigInteger != null) {
                this.u = this.u.modPow(bigInteger, bigInteger3);
            }
            else {
                this.v = this.v.modPow(bigInteger2, bigInteger3);
            }
        }
        
        RSACore.BlindingRandomPair getBlindingRandomPair(BigInteger x, BigInteger x2, BigInteger bigInteger) {
            if ((this.e != null && this.e.equals(x)) || (this.d != null && this.d.equals(x2))) {
                RSACore.BlindingRandomPair blindingRandomPair = null;
                synchronized (this) {
                    if (!this.u.equals(BigInteger.ZERO) && !this.v.equals(BigInteger.ZERO)) {
                        blindingRandomPair = new RSACore.BlindingRandomPair(this.u, this.v);
                        if (this.u.compareTo(BigInteger.ONE) <= 0 || this.v.compareTo(BigInteger.ONE) <= 0) {
                            this.u = BigInteger.ZERO;
                            this.v = BigInteger.ZERO;
                        }
                        else {
                            this.u = this.u.modPow(RSACore.BlindingParameters.BIG_TWO, bigInteger);
                            this.v = this.v.modPow(RSACore.BlindingParameters.BIG_TWO, bigInteger);
                        }
                    }
                }
                return blindingRandomPair;
            }
            return null;
        }
        
        static {
            BIG_TWO = BigInteger.valueOf(2L);
        }
    }
}
